#====================================================================================
#Goal: 
# determine the probability that the 'true' median response to Canaki is better than
# that of Abata and Adali, at week 12 and 52
# Because we want to look at the 'true' median response eta_BTAV=0, eta_BSV=0 and a=0
#
#====================================================================================

library(mlxR)
library(gridExtra)
library(lixoftConnectors)
# + set the working directory to the file location

source("./getPopUncertainty.R")

#====================================================================
#  1 simulation using the estimated population parameters
#====================================================================

# model without study and arm random effects, and without residual error
ACR20model <-inlineModel("
                      [LONGITUDINAL]
                      input = {Emax, T50}
                      
                      EQUATION:
                      ACR20 = Emax * (t / (T50 + t))
                      
                      [COVARIATE]
                      input = {tDRUG}
                      tDRUG = {type=categorical, categories={abata, adali, canaki, placebo}}
                      
                      [INDIVIDUAL]
                      input = {Emax_pop, beta_Emax_tDRUG_abata, beta_Emax_tDRUG_adali, beta_Emax_tDRUG_canaki, tDRUG, T50_pop, beta_T50_tDRUG_abata, beta_T50_tDRUG_adali, beta_T50_tDRUG_canaki}
                      tDRUG = {type=categorical, categories={abata, adali, canaki, placebo}}
                      
                      DEFINITION:
                      Emax = {distribution=logitnormal, typical=Emax_pop, covariate=tDRUG, coefficient={beta_Emax_tDRUG_abata, beta_Emax_tDRUG_adali, beta_Emax_tDRUG_canaki, 0}, no-variability}
                      T50 = {distribution=lognormal, typical=T50_pop, covariate=tDRUG, coefficient={beta_T50_tDRUG_abata, beta_T50_tDRUG_adali, beta_T50_tDRUG_canaki, 0}, no-variability}
                      
                      ")

# get population parameters from populationParameters.txt (file name for 2018 and 2019 Monolix version)
populationParameter <- read.table('./monolix_project/final_project/populationParameters.txt', header=TRUE, sep=',')
param <- populationParameter[,"value"]
names(param) <- populationParameter[,"parameter"]


# define what we want to look at
out <- list(name='ACR20',time=seq(0,52,by=1))

# define the tDRUG covariate: we will do 4 simulations, one for each treatment
covariates <- data.frame(id=1:4,
                         tDRUG=c('abata','adali','canaki','placebo'))

#simulation
res <- simulx(model=ACR20model,
              parameter=list(covariates,param),
              output = out)

#experimental data on canaki
timeexpe <- c(2,4,8,12)
canakexpe<- c(0.2298, 0.3885, 0.4172, 0.4896)
df_expecanak <- data.frame(timeexpe, canakexpe)

#plotting
p1 <- ggplot() +
  geom_line(data=res$ACR20, aes(x=time, y=ACR20, colour=id), size=2) +
  ylab("Fraction of patients with ACR20")+xlab("Time (weeks)")+
  theme_bw() +
  geom_point(data=df_expecanak, aes(x=timeexpe, y=canakexpe), colour="#E69F00", size=3)+
  scale_colour_manual(values=c("#56B4E9","#0072B2", "#E69F00","#999999"),
                      name  ="Treatment",
                      labels=c("Abata", "Adali","Canaki","Placebo")) +
  ylim(0, 0.8) +
  theme(legend.justification=c(1,0), legend.position=c(1,0))+
  theme(axis.title.x = element_text(size=14),
        axis.text.x  = element_text(size=14),
        axis.title.y = element_text(size=14),
        axis.text.y  = element_text(size=14),
        legend.title = element_text(size=14),
        legend.text  = element_text(size=14))

print(p1)


#================================================================================
#     Taking into account the uncertainty in the population parameters
#================================================================================

# number of studies simulated for each of the three drugs, to take into account the uncertainty
NBstudy <- 500

# we will use simpopmlx to draw population parameters, taking into account the correlations
paramWithUncertainty <- getPopUncertainty(project='./monolix_project/final_project.mlxtran', nRep=4*NBstudy)
colnames(paramWithUncertainty)[1] <- "id" 

# define outputs
out <- list(name='ACR20',time=seq(0,52,by=1))
outc<- list(name='tDRUG')

# define covariates
covariates <- data.frame(id=1:(4*NBstudy),
                         tDRUG=rep(c('abata','adali','canaki','placebo'),each=NBstudy))

#simulation
res2 <- simulx(model=ACR20model,
               parameter=list(covariates,paramWithUncertainty),
               output = list(out,outc))

# re-arranging the data frames
df <- merge(x=res2$ACR20, y=res2$parameter, by='id', all=TRUE)
dfAbata <- subset(df, tDRUG=='abata')
dfAbata$tDRUG <- NULL
dfAdali <- subset(df, tDRUG=='adali')
dfAdali$tDRUG <- NULL
dfCanak <- subset(df, tDRUG=='canaki')
dfCanak$tDRUG <- NULL
dfPlace <- subset(df, tDRUG=='placebo')
dfPlace$tDRUG <- NULL

#calculating the probability
AbataW12 <- dfAbata[dfAbata$time==12,3]
AdaliW12 <- dfAdali[dfAdali$time==12,3]
CanakW12 <- dfCanak[dfCanak$time==12,3]
PercentCanakiBetterAbataW12= sum(CanakW12>AbataW12)/length(AbataW12)*100
PercentCanakiBetterAdaliW12= sum(CanakW12>AdaliW12)/length(AdaliW12)*100
paste0('At week 12, there is ',PercentCanakiBetterAbataW12,' percent chances that Canaki is better than Abata and ',PercentCanakiBetterAdaliW12,' percent chances it is better than Adali') 

AbataW52 <- dfAbata[dfAbata$time==52,3]
AdaliW52 <- dfAdali[dfAdali$time==52,3]
CanakW52 <- dfCanak[dfCanak$time==52,3]
PercentCanakiBetterAbataW52= sum(CanakW52>AbataW52)/length(AbataW52)*100
PercentCanakiBetterAdaliW52= sum(CanakW52>AdaliW52)/length(AdaliW52)*100
paste0('At week 52, there is ',PercentCanakiBetterAbataW52,' percent chances that Canaki is better than Abata and ',PercentCanakiBetterAdaliW52,' percent chances it is better than Adali')

# calculating the prediction percentiles for plotting
pctAbata <- prctilemlx(dfAbata, band=list(number=1,level=90), plot=FALSE)
pctAdali <- prctilemlx(dfAdali, band=list(number=1,level=90), plot=FALSE)
pctCanak <- prctilemlx(dfCanak, band=list(number=1,level=90), plot=FALSE)
pctPlace <- prctilemlx(dfPlace, band=list(number=1,level=90), plot=FALSE)

# renaming
names(pctAbata$y)<-c('time','p5','p50','p95')
names(pctAdali$y)<-c('time','p5','p50','p95')
names(pctCanak$y)<-c('time','p5','p50','p95')
names(pctPlace$y)<-c('time','p5','p50','p95')
pctAbata$y$drug<-'Abata'
pctAdali$y$drug<-'Adali'
pctCanak$y$drug<-'Canaki'
pctPlace$y$drug<-'Placebo'


#plotting the prediction intervals
pctA<-rbind(pctAbata$y,pctCanak$y)
pA <- ggplot()+
  geom_ribbon(data=pctA,aes(x=time,ymin=p5, ymax=p95,fill=drug),alpha=.4)+
  geom_line(data=pctA,aes(x=time,y=p50,colour=drug))+
  #geom_point(data=df_expecanak, aes(x=timeexpe, y=canakexpe), colour="#E69F00")+
  scale_y_continuous(limits=c(0,0.85))+
  ylab("Fraction of patients with ACR20")+xlab("Weeks")+
  theme_bw() +
  scale_fill_manual(values=c("#56B4E9", "#E69F00"), name="Treatment") +
  scale_colour_manual(values=c("#56B4E9", "#E69F00"), name='Treatment') +
  guides(colour=FALSE)+
  ggtitle("Abata vs Canaki") +
  theme(legend.justification=c(1,0), legend.position=c(1,0))+
  theme(axis.title.x = element_text(size=14),
        axis.text.x  = element_text(size=14),
        axis.title.y = element_text(size=14),
        axis.text.y  = element_text(size=14),
        legend.title = element_text(size=14),
        legend.text  = element_text(size=14),
        plot.title   = element_text(size=14, face='bold', hjust=0.5))

pctB<-rbind(pctCanak$y,pctAdali$y)
pB <- ggplot(data=pctB)+
  geom_ribbon(aes(x=time,ymin=p5, ymax=p95,fill=drug), alpha=.4)+
  geom_line(aes(x=time,y=p50,colour=drug))+
  scale_y_continuous(limits=c(0,0.85))+
  ylab("Fraction of patients with ACR20")+xlab("Weeks")+
  theme_bw() +
  scale_fill_manual(values=c("#0072B2", "#E69F00"), name="Treatment") +
  scale_colour_manual(values=c("#0072B2", "#E69F00"), name='Treatment') +
  guides(colour=FALSE)+
  ggtitle("Adali vs Canaki") +
  theme(legend.justification=c(1,0), legend.position=c(1,0))+
  theme(axis.title.x = element_text(size=14),
        axis.text.x  = element_text(size=14),
        axis.title.y = element_text(size=14),
        axis.text.y  = element_text(size=14),
        legend.title = element_text(size=14),
        legend.text  = element_text(size=14),
        plot.title   = element_text(size=14, face='bold', hjust=0.5))

print(pB)
grid.arrange(pA,pB,ncol=2)