#====================================================================================
#Goal: determine the probability that in a trial comparing Placebo and Canaki at week 52,
#      canaki turns out to be better. Calculate power of the study for Narm=50,100,200 or 400
#
# => simulate clinical trials
#====================================================================================

library(mlxR)
library(gridExtra)
library(lixoftConnectors)
# + set working directory to file location

source("./getPopUncertainty.R")

#====================================================================
#  1 clinical trial simulation:
# - calculate the variance using a formula
# - test if Canaki if significantly better than Placebo
#====================================================================

# define model (with BSV, BTAV and residual error)
myModel <-inlineModel("
                      [LONGITUDINAL]
                      input = {Emax, T50, etaBSVEmax, etaBTAVEmax_OCCcanak,etaBTAVEmax_OCCplace, Narm, ARM, a}
                      Narm = {use=regressor}
                      ARM   = {use=regressor}
                      
                      EQUATION:
                      eta_BTAV=0
                      if ARM==0 ;ARM=0 corresponds to placebo
                      etaBTAVEmax = etaBTAVEmax_OCCplace
                      end 
                      if ARM==1 ;ARM=0 corresponds to canaki
                      etaBTAVEmax = etaBTAVEmax_OCCcanak
                      end 
                      
                      ; transform Emax to tEmax (normally distributed) 
                      tEmax = logit(Emax)

                      
                      ; adding the random effects (RE) due to between-study variability and between arm variability, on the transformed parameters 
                      tEmaxRE = tEmax + etaBSVEmax + etaBTAVEmax/sqrt(Narm)
                      
                      ; transforming back to have EmaxRE with logit distribution (values between 0 and 1)
                      EmaxRE = 1 / (1+ exp(-tEmaxRE) )

                      
                      ; defining the effect
                      ACR20 = EmaxRE * (t / (T50 + t))
                      
                      ; adding a saturation to avoid taking logit(0) (undefined) when t=0
                      ACR20sat = max(ACR20, 0.000001)
                      
                      ; transforming the effect ACR20 in the same way as the data
                      pred = logit(ACR20sat)*sqrt(Narm)
                      
                      DEFINITION:
                      y= {distribution=normal, prediction=pred, sd=a}
                      
                      [COVARIATE]
                      input = {tDRUG}
                      tDRUG = {type=categorical, categories={placebo, canaki}}
                      
                      [INDIVIDUAL]
                      input = {Emax_pop, beta_Emax_tDRUG_canaki, tDRUG, T50_pop, beta_T50_tDRUG_canaki, omega_etaBSVEmax, gamma1_etaBTAVEmax}
                      tDRUG = {type=categorical, categories={placebo, canaki}}
                      
                      DEFINITION:
                      Emax = {distribution=logitnormal, typical=Emax_pop, covariate=tDRUG, coefficient={0, beta_Emax_tDRUG_canaki}, no-variability}
                      T50 = {distribution=lognormal, typical=T50_pop, covariate=tDRUG, coefficient={0, beta_T50_tDRUG_canaki}, no-variability}
                      etaBSVEmax = {distribution=normal, typical=0, sd=omega_etaBSVEmax}
                      etaBTAVEmax_OCCcanak = {distribution=normal, typical=0, sd=gamma1_etaBTAVEmax}
                      etaBTAVEmax_OCCplace = {distribution=normal, typical=0, sd=gamma1_etaBTAVEmax}
                      ")


# get population parameters from populationParameters.txt
populationParameter <- read.table('./monolix_project/final_project/populationParameters.txt', header=TRUE, sep=',')
param <- populationParameter[,"value"]
names(param) <- populationParameter[,"parameter"]


# number of studies simulated for each of placebo and canaki
NBstudy <- 1

# number of people in each arm
Narm=200

# generate population parameter from the correlation matrix
paramWithUncertainty <- getPopUncertainty(project='./monolix_project/final_project.mlxtran', nRep=2*NBstudy)
colnames(paramWithUncertainty)[1] <- "id" 

# define outputs: times at which we do a measurement
out <- list(name='y',time=c(2,4,8,12,24,52))
outc<- list(name='tDRUG')

# define covariates
covariates <- data.frame(id=1:(2*NBstudy),
                         tDRUG=rep(c('placebo','canaki'),each=NBstudy))

# defining the regressors
regtimes <- c(0,52)
reg <- data.frame(id = rep(1:(2*NBstudy),each=length(regtimes)),
                  time = rep(regtimes,NBstudy*2),
                  Narm = Narm,
                  ARM = c(rep(0,length(regtimes)*NBstudy),rep(1,length(regtimes)*NBstudy)))

#simulation
res3 <- simulx(model=myModel,
               parameter=list(covariates,param),
               output = list(out,outc),
               regressor = reg)

#transform the observation y (in [-Inf,+Inf]) back to ACR20 observation (in [0,1])
res3$y$ObsEffect <- 1/(1+exp(-1*res3$y$y/sqrt(Narm)))

#calculate the standard deviation 
res3$y$sd <- sqrt(1/Narm * res3$y$ObsEffect * (1-res3$y$ObsEffect))


#plotting
p1 <- ggplot() +
  geom_errorbar(data=res3$y, aes(x=time, ymin=ObsEffect-sd, ymax=ObsEffect+sd, colour=id), width=1) +
  geom_point(data=res3$y, aes(x=time, y=ObsEffect, colour=id), size=2) +
  geom_line(data=res3$y, aes(x=time, y=ObsEffect, colour=id), size=0.5) +
  ylab("Fraction of patients with ACR20")+xlab("Time (weeks)")+
  theme_bw() +
  scale_colour_manual(values=c("#999999", "#E69F00"),
                      name  ="Treatment",
                      labels=c("Placebo","Canaki")) +
  theme(axis.title.x = element_text(size=14),
        axis.text.x  = element_text(size=14),
        axis.title.y = element_text(size=14),
        axis.text.y  = element_text(size=14),
        legend.title = element_text(size=14),
        legend.text  = element_text(size=14))

print(p1)

#focus on week 52
w52=res3$y[res3$y$time==52,]

# statistical test for proportion at week 52
NbPlacebo=w52$ObsEffect[1]*Narm
NbCanaki=w52$ObsEffect[2]*Narm
prop.test(x=c(NbPlacebo,NbCanaki),n=c(Narm,Narm),alternative="less")


#======================================================================
# Simulate a large number of studies to calculate the power of the study
#======================================================================

# number of studies simulated for each of the two treatments
NBstudy <- 1000

# define outputs
out <- list(name='y',time=c(2,4,8,12,24,52))
outc<- list(name='tDRUG')

# generate population parameter from the correlation matrix
paramWithUncertainty <- getPopUncertainty(project='./monolix_project/final_project.mlxtran', nRep=2*NBstudy)
colnames(paramWithUncertainty)[1] <- "id" 

# define covariates
covariates <- data.frame(id=1:(2*NBstudy),
                         tDRUG=rep(c('placebo','canaki'),each=NBstudy))

# defining the regressors
regtimes <- c(0,52)

# number of people per arm: testing several values
Narm <- c(50,100,200,400)

# create empty vector for the power of the study
PowerStudy <-c(0,0,0,0)

# for each number of patients per arm
for(i in 1:length(Narm)){
  
  #define regressors
  reg <- data.frame(id = rep(1:(2*NBstudy),each=length(regtimes)),
                    time = rep(regtimes,NBstudy*2),
                    Narm = Narm[i],
                    ARM = c(rep(0,length(regtimes)*NBstudy),rep(1,length(regtimes)*NBstudy)))
  
  #simulation
  res4 <- simulx(model=myModel,
                 parameter=list(covariates,paramWithUncertainty),
                 output = list(out,outc),
                 regressor = reg)
  
  # calculate ACR20 from observation
  res4$y$ObsEffect <- 1/(1+exp(-1*res4$y$y/sqrt(Narm[i])))
  
  # re-arranging the data frames, and selecting week 52
  df <- merge(x=res4$y, y=res4$parameter, by='id', all=TRUE)
  dfCanak <- subset(df, tDRUG=='canaki' & time==52)
  dfCanak$tDRUG <- NULL
  dfCanak$y <- NULL
  dfPlace <- subset(df, tDRUG=='placebo' & time==52)
  dfPlace$tDRUG <- NULL
  dfPlace$y <- NULL
  
  # from all simulated studies, calculate the fraction that would result in a successful trial (significant difference between placebo and canaki)
  nbSuccessFullTrials=0
  for(k in 1:length(dfCanak$id)){ # for each study
    tt <- prop.test(x=c(dfPlace$ObsEffect[k],dfCanak$ObsEffect[k])*Narm[i],n=rep(Narm[i],2), alternative="less")
    if(tt$p.value<0.05){
      nbSuccessFullTrials=nbSuccessFullTrials+1
    }
  }
  PowerStudy[i]=nbSuccessFullTrials/NBstudy
  print(paste0("For Narm=",Narm[i],", the power of the study would be ", PowerStudy[i]))
}


