####################################################################################################
# Get the covariates used in the project
###################################################################################################
getPopUncertainty = function(projectFile, nRep){
  initializeLixoftConnectors(software = "monolix", force = T)
  loadProject(projectFile)
  #----------------------------------------------------------------------------------------
  # Prepare the population parameters
  param <- getPopulationParameterInformation()
  # Remove teh fixed parameters
  indexFixed = which(param$method == 'FIXED' )
  if(length(indexFixed)>0){
    param <- param[-indexFixed,]
  }
  
  param$pop.param = getEstimatedPopulationParameters()
  param$sd = getEstimatedStandardErrors()[[1]]
  param$trans = rep('L', length(param[,1]))
  param$bound_min = rep(0, length(param[,1]))
  param$bound_max = rep(Inf, length(param[,1]))
  
  # Update the distribution
  for(indexParam in 1:length(param$name)){
    paramName = param$name[indexParam]
    if(nchar(paramName)>4){
      endString = substr(paramName, nchar(paramName)-4+1, nchar(paramName))
      if(endString=='_pop'){
        paramName_pop = substr(paramName, 1, nchar(paramName)-4)
        distribution = getIndividualParameterModel()$distribution[which(paramName_pop==getIndividualParameterModel()$name)]
        if(length(distribution)>0){
          if(tolower(distribution)=='normal'){
            param$trans[indexParam] = 'N';
            param$bound_min[indexParam] = -Inf
          }else if(tolower(distribution)=='logitnormal'){
            param$trans[indexParam] = 'G'
            paramLimits = getIndividualParameterModel()$limits[[paramName_pop]] 
            param$bound_min[indexParam] = paramLimits[1]
            param$bound_max[indexParam] = paramLimits[2]
          }else if(tolower(distribution)=='probitnormal'){
            param$trans[indexParam] = 'P'
            param$bound_min[indexParam] = 0
            param$bound_max[indexParam] = 1
          }
        }
      }
    }
    
    if(grepl(x = paramName, pattern = 'beta_')){
      param$trans[indexParam] = 'N'
      param$bound_min[indexParam] = -Inf
    } 
    
    if(grepl(x = paramName, pattern = 'corr_')||grepl(x = paramName, pattern = 'corr1_')||grepl(x = paramName, pattern = 'corr2_')){
      param$trans[indexParam] = 'G'
      param$bound_min[indexParam] = 0
      param$bound_max[indexParam] = 1
    }
  }
  
  #----------------------------------------------------------------------------------------
  # Prepare the correlation
  # read correlation between parameter estimates
  corr <- read.csv(file=paste0(getProjectSettings()$directory,'/FisherInformation/correlationEstimatesSA.txt'),
                   header=F)
  corr <- corr[,-1]
  
  paramWithUncertainty <- .simpopmlx(n=nRep, parameter = param, corr=corr)
  
  initializeLixoftConnectors(software = "simulx", force = T)
  return(paramWithUncertainty)
}

####################################################################################################
# Simulate parameters based on their distribution and the correlation matrix
###################################################################################################
.simpopmlx = function (n = 1, parameter = NULL, corr = NULL, kw.max = 100){
  
  mu <- parameter$pop.param
  sd <- parameter$sd
  trans <- parameter$trans
  #pname <- row.names(parameter)
  pname <- parameter$name
  np <- length(mu)
  bound_min =  parameter$bound_min
  bound_max = parameter$bound_max
  
  i.corr <- unique(c(grep("^r_", pname), grep("^corr_", pname)))
  
  if (is.null(corr)) {
    d <- rep(1, np)
    corr <- diag(d)
    isd0 <- which(sd == 0)
    if (length(isd0) > 0) {
      corr[isd0, ] <- NaN
      corr[, isd0] <- NaN
    }
  }
  inan <- which(is.nan(as.matrix(corr)), arr.ind = TRUE)
  if (dim(inan)[1] > 0) 
    i1 <- which(as.vector(table(inan[, 1])) < np)
  else i1 <- (1:np)
  if (!is.null(corr)) 
    corr1 <- corr[i1, i1]
  else corr1 <- diag(rep(1, length(i1)))
  if (length(which(is.na(corr1))) > 0) 
    stop("ERROR:  the correlation matrix of the estimates contains NaN")
  se1 <- sd[i1]
  tr1 <- trans[i1]
  mu1 <- mu[i1]
  set <- se1
  mut <- mu1
  iL <- which(tr1 == "L")
  set[iL] <- se1[iL]/mu1[iL]
  mut[iL] <- log(mu1[iL])
  
  iG <- which(tr1 == "G")
  mut[iG] <- log((mu1[iG]-bound_min[iG])/(bound_max[iG] - mu1[iG]))
  set[iG] <- se1[iG]*(bound_max[iG]-bound_min[iG])/((bound_max[iG]-mu1[iG]) * (mu1[iG]-bound_min[iG]))
  
  iR <- which(tr1 == "R")
  set[iR] <- se1[iR] * 2/(1 - mu1[iR]^2)
  mut[iR] <- log((mu1[iR] + 1)/(1 - mu1[iR]))
  iP <- which(tr1 == "P")
  set[iP] <- se1[iP]/dnorm(qnorm(mu1[iP]))
  mut[iP] <- qnorm(mu1[iP])
  Rt <- chol((set %*% t(set)) * corr1)
  K <- length(i1)
  n.corr <- length(i.corr)
  if (n.corr > 0) {
    corr.name <- pname[i.corr]
    g = gregexpr("_", corr.name)
    nk1 <- vector(length = n.corr)
    nk2 <- vector(length = n.corr)
    for (k in (1:n.corr)) {
      if (length(g[[k]]) > 2) 
        stop("ERROR :  you should not use parameter names with \"_\" ")
      cnk <- corr.name[k]
      gk <- g[[k]][1:2]
      nk1[k] <- substr(cnk, gk[1] + 1, gk[2] - 1)
      nk2[k] <- substr(cnk, gk[2] + 1, nchar(cnk))
    }
    nvar <- unique(c(nk1, nk2))
    ir1 <- match(nk1, nvar)
    ir2 <- match(nk2, nvar)
    ind1.r <- matrix(c(ir2, ir1), ncol = 2)
    ind2.r <- matrix(c(ir1, ir2), ncol = 2)
    n.r <- length(nvar)
    R.r <- diag(rep(1, n.r))
  }
  n1 <- n
  s.res <- NULL
  kw <- 0
  while (n1 > 0) {
    kw <- kw + 1
    if (kw > kw.max) 
      stop("Maximum number of iterations reached: could not draw definite positive correlation matrix")
    x = matrix(rnorm(K * n1), ncol = K)
    st <- t(t(x %*% Rt) + mut)
    st[, iL] <- exp(st[, iL])
    #st[, iG] <- 1/(1 + exp(-st[, iG]))
    st[, iG] <- (bound_max[iG]*exp(st[, iG])+bound_min[iG])/(exp(st[, iG])+1)
    
    st[, iP] <- pnorm(st[, iP])
    st[, iR] <- (exp(st[, iR]) - 1)/(exp(st[, iR]) + 1)
    s <- t(replicate(n1, mu))
    s[, i1] <- st
    if (n.corr > 0) {
      for (i in (1:n1)) {
        sri <- s[i, i.corr]
        R.r[ind1.r] <- sri
        R.r[ind2.r] <- sri
        R.eig <- eigen(R.r, symmetric = TRUE, only.values = TRUE)
        if (min(R.eig$values) > 0) {
          n1 <- n1 - 1
          s.res <- rbind(s.res, s[i, ])
        }
      }
    }
    else {
      n1 <- 0
      s.res <- s
    }
  }
  s.res <- as.data.frame(s.res)
  names(s.res) <- pname
  if (nrow(s.res) > 1) 
    s.res <- cbind(list(pop = (1:nrow(s.res))), s.res)
  return(s.res)
}
